@@ -99,6 +99,7 @@ struct _GVfsBackendTrash
 
   /* This is only set on the main thread */
   GList *top_files; /* Files in toplevel dir */
+  guint num_top_files;
 
   /* All these are protected by the root_monitor lock */
   GVfsMonitor *file_vfs_monitor;
@@ -112,12 +113,13 @@ struct _GVfsBackendTrash
 
 G_LOCK_DEFINE_STATIC(root_monitor);
 
-G_DEFINE_TYPE (GVfsBackendTrash, g_vfs_backend_trash, G_VFS_TYPE_BACKEND);
+G_DEFINE_TYPE (GVfsBackendTrash, g_vfs_backend_trash, G_VFS_TYPE_BACKEND)
 
-static void   schedule_update_trash_files (GVfsBackendTrash *backend,
-                                           gboolean          update_trash_dirs);
-static GList *enumerate_root              (GVfsBackend      *backend,
-                                           GVfsJobEnumerate *job);
+static void   schedule_update_trash_files  (GVfsBackendTrash *backend,
+                                            gboolean          update_trash_dirs);
+static GList *enumerate_root               (GVfsBackend      *backend,
+                                            GVfsJobEnumerate *job);
+static GVfsMonitor *do_create_root_monitor (GVfsBackend      *backend);
 
 static char *
 escape_pathname (const char *dir)
@@ -200,8 +202,9 @@ unescape_pathname (const char *escaped_dir, int len)
     {
       char *trashname;
       trashname = g_strndup (escaped_dir + 1, len - 1);
-      return g_build_filename (g_get_user_data_dir (), "Trash", trashname, NULL);
+      dir = g_build_filename (g_get_user_data_dir (), "Trash", trashname, NULL);
       g_free (trashname);
+      return dir;
     }
   
   dir = g_malloc (len + 1 + 1);
@@ -252,7 +255,7 @@ get_top_dir_for_trash_dir (const char *trash_dir)
       return g_path_get_dirname (trash_dir);
     }
   
-  user_trash_basename =  g_strdup_printf (".Trash-%d", getuid());
+  user_trash_basename =  g_strdup_printf (".Trash-%u", getuid());
   if (strcmp (basename, user_trash_basename) == 0)
     {
       g_free (user_trash_basename);
@@ -261,7 +264,7 @@ get_top_dir_for_trash_dir (const char *trash_dir)
     }
   g_free (user_trash_basename);
 
-  user_sys_dir = g_strdup_printf ("%d", getuid());
+  user_sys_dir = g_strdup_printf ("%u", getuid());
   if (strcmp (basename, user_sys_dir) == 0)
     {
       g_free (user_sys_dir);
@@ -346,7 +349,7 @@ check_topdir (const char *topdir)
       statbuf.st_mode & S_ISVTX)
     {
       /* We have a valid sysadmin .Trash dir, look for uid subdir */
-      sysadmin_dir_uid = g_strdup_printf ("%s/%d", sysadmin_dir, getuid());
+      sysadmin_dir_uid = g_strdup_printf ("%s/%u", sysadmin_dir, getuid());
       
       if (lstat (sysadmin_dir_uid, &statbuf) == 0 &&
           S_ISDIR (statbuf.st_mode) &&
@@ -362,7 +365,7 @@ check_topdir (const char *topdir)
     }
   g_free (sysadmin_dir);
 
-  user_trash_basename =  g_strdup_printf (".Trash-%d", getuid());
+  user_trash_basename =  g_strdup_printf (".Trash-%u", getuid());
   user_trash = g_build_filename (topdir, user_trash_basename, NULL);
   g_free (user_trash_basename);
   
@@ -552,7 +555,7 @@ list_trash_dirs (void)
 
       if (info & HAS_SYSTEM_DIR)
         {
-          basename = g_strdup_printf ("%d", getuid());
+          basename = g_strdup_printf ("%u", getuid());
           trashdir = g_build_filename (topdir, ".Trash", basename, NULL);
           g_free (basename);
           dirs = g_list_prepend (dirs, trashdir);
@@ -560,7 +563,7 @@ list_trash_dirs (void)
       
       if (info & HAS_USER_DIR)
         {
-          basename = g_strdup_printf (".Trash-%d", getuid());
+          basename = g_strdup_printf (".Trash-%u", getuid());
           trashdir = g_build_filename (topdir, basename, NULL);
           g_free (basename);
           dirs = g_list_prepend (dirs, trashdir);
@@ -570,6 +573,10 @@ list_trash_dirs (void)
         has_trash_files = TRUE;
     }
 
+  g_list_foreach (topdirs, (GFunc) g_free, NULL);
+  g_list_free (topdirs);
+  g_list_free (topdirs_info);
+
   return g_list_reverse (dirs);
 }
 
@@ -614,7 +621,9 @@ do_mount (GVfsBackend *backend,
   GList *names;
 
   names = enumerate_root (backend, NULL);
+  trash_backend->num_top_files = g_list_length (names);
   trash_backend->top_files = g_list_sort (names, (GCompareFunc)strcmp);
+  do_create_root_monitor (backend);
   
   g_vfs_job_succeeded (G_VFS_JOB (job));
  }
@@ -666,7 +675,6 @@ do_open_for_read (GVfsBackend *backend,
     }
 }
 
-
 static void
 do_read (GVfsBackend *backend,
          GVfsJobRead *job,
@@ -747,7 +755,6 @@ do_close_read (GVfsBackend *backend,
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
       g_error_free (error);
     }
-
 }
 
 typedef struct {
@@ -758,18 +765,23 @@ typedef struct {
 static gboolean
 set_trash_files (gpointer _data)
 {
+  GVfsMonitor *vfs_monitor, *file_vfs_monitor;
   GVfsBackendTrash *trash_backend;
   SetHasTrashFilesData *data = _data;
-  GList *new, *old, *l;
-  int cmp;
-  char *name;
-  GList *added;
-  GList *removed;
-  GVfsMonitor *vfs_monitor, *file_vfs_monitor;
+  GList *new_list, *old_list;
+  guint old_count, new_count;
 
   trash_backend = G_VFS_BACKEND_TRASH (data->backend);
-  
-  data->names = g_list_sort (data->names, (GCompareFunc)strcmp);
+  new_list = g_list_sort (data->names, (GCompareFunc)strcmp);
+  g_slice_free (SetHasTrashFilesData, data);
+
+  old_list = trash_backend->top_files;
+  old_count = trash_backend->num_top_files;
+
+  /* do the replacement now, before we send notifications.  */
+  new_count = g_list_length (new_list);
+  trash_backend->num_top_files = new_count;
+  trash_backend->top_files = new_list;
  
   G_LOCK (root_monitor);
   vfs_monitor = NULL;
@@ -783,11 +795,16 @@ set_trash_files (gpointer _data)
 
   if (vfs_monitor)
     {
+      GList *added, *removed;
+      GList *new, *old, *l;
+      char *name;
+      int cmp;
+
       added = NULL;
       removed = NULL;
-      
-      old = trash_backend->top_files;
-      new = data->names;
+
+      new = new_list;
+      old = old_list;
       
       while (new != NULL || old != NULL)
         {
@@ -844,10 +861,10 @@ set_trash_files (gpointer _data)
         }
       g_list_free (added);
       
-      if ((trash_backend->top_files == NULL && data->names != NULL) ||
-          (trash_backend->top_files != NULL && data->names == NULL))
+      if (new_count != old_count)
         {
-          /* "fullness" changed => icon change */
+          /* trash::item-count changed. */
+          /* icon change only ever occurs when item-count also changes */
           g_vfs_monitor_emit_event (vfs_monitor,
                                     G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED,
                                     "/",
@@ -859,10 +876,10 @@ set_trash_files (gpointer _data)
 
   if (file_vfs_monitor)
     {
-      if ((trash_backend->top_files == NULL && data->names != NULL) ||
-          (trash_backend->top_files != NULL && data->names == NULL))
+      if (new_count != old_count)
         {
-          /* "fullness" changed => icon change */
+          /* trash::item-count changed. */
+          /* icon change only ever occurs when item-count also changes */
           g_vfs_monitor_emit_event (file_vfs_monitor,
                                     G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED,
                                     "/",
@@ -872,12 +889,8 @@ set_trash_files (gpointer _data)
       g_object_unref (file_vfs_monitor);
     }
 
-  g_list_foreach (trash_backend->top_files, (GFunc)g_free, NULL);
-  g_list_free (trash_backend->top_files);
-
-  trash_backend->top_files = data->names;
-  
-  g_free (data);
+  g_list_foreach (old_list, (GFunc)g_free, NULL);
+  g_list_free (old_list);
   
   return FALSE;
 }
@@ -887,7 +900,7 @@ queue_set_trash_files (GVfsBackend *backend,
 {
   SetHasTrashFilesData *data;
 
-  data = g_new (SetHasTrashFilesData, 1);
+  data = g_slice_new (SetHasTrashFilesData);
   data->backend = backend;
   data->names = names;
   g_idle_add (set_trash_files, data);
@@ -940,9 +953,14 @@ add_extra_trash_info (GFileInfo *file_info,
 
           if (orig_path_unescaped)
             {
-              /* Set display name and edit name based of original basename */
-              display_name = g_filename_display_basename (orig_path_unescaped);
+              if (g_path_is_absolute (orig_path_unescaped))
+                orig_path = g_build_filename (orig_path_unescaped, relative_path, NULL);
+              else
+                orig_path = g_build_filename (topdir, orig_path_unescaped, relative_path, NULL);
+              g_free (orig_path_unescaped);
 
+              /* Set display name and edit name based of original basename */
+              display_name = g_filename_display_basename (orig_path);
               g_file_info_set_edit_name (file_info, display_name);
               
               if (strstr (display_name, "\357\277\275") != NULL)
@@ -964,22 +982,11 @@ add_extra_trash_info (GFileInfo *file_info,
               
               g_free (display_name);
               
-              
-              /* Set orig_path */
-              
-              if (g_path_is_absolute (orig_path_unescaped))
-                orig_path = g_build_filename (orig_path_unescaped, relative_path, NULL);
-              else
-                orig_path = g_build_filename (topdir, orig_path_unescaped, relative_path, NULL);
-
-              
               g_file_info_set_attribute_byte_string (file_info,
                                                      "trash::orig-path",
                                                      orig_path);
               g_free (orig_path);
-              g_free (orig_path_unescaped);
             }
-          
 
           g_free (orig_path_key);
         }
@@ -1004,9 +1011,6 @@ enumerate_root_trashdir (GVfsBackend *backend,
 { 
   GFile *file, *files_file;
   GFileEnumerator *enumerator;
-  GFileInfo *info;
-  const char *name;
-  char *new_name, *new_name_escaped;
   char *info_dir;
 
   info_dir = g_build_filename (trashdir, "info", NULL);
@@ -1024,10 +1028,15 @@ enumerate_root_trashdir (GVfsBackend *backend,
 
   if (enumerator)
     {
+      GFileInfo *info;
+
       while ((info = g_file_enumerator_next_file (enumerator,
                                                   job ? G_VFS_JOB (job)->cancellable : NULL,
                                                   NULL)) != NULL)
         {
+          const char *name;
+          char *new_name, *new_name_escaped;
+
           name = g_file_info_get_name (info);
 
           /* Get the display name, etc */
@@ -1037,7 +1046,6 @@ enumerate_root_trashdir (GVfsBackend *backend,
                                 name,
                                 NULL);
 
-          
           /* Update the name to also have the trash dir */
           new_name = g_build_filename (trashdir, name, NULL);
           new_name_escaped = escape_pathname (new_name);
@@ -1056,6 +1064,8 @@ enumerate_root_trashdir (GVfsBackend *backend,
                                NULL);
       g_object_unref (enumerator);
     }
+
+  g_free (info_dir);
 }
 
 static GList *
@@ -1110,7 +1120,6 @@ do_enumerate (GVfsBackend *backend,
       GFile *file;
       GFileEnumerator *enumerator;
       GFileInfo *info;
-      const char *name;
       char *dir;
       GError *error;
 
@@ -1134,8 +1143,6 @@ do_enumerate (GVfsBackend *backend,
                                                       G_VFS_JOB (job)->cancellable,
                                                       NULL)) != NULL)
             {
-              name = g_file_info_get_name (info);
-              
               g_vfs_job_enumerate_add_info   (job, info);
               g_object_unref (info);
             }
@@ -1173,6 +1180,8 @@ do_query_info (GVfsBackend *backend,
 
   if (!decode_path (filename, &trashdir, &trashfile, &relative_path, &topdir))
     {
+      GVfsBackendTrash *trash_backend = G_VFS_BACKEND_TRASH (backend);
+
       /* The trash:/// root */
       g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
       g_file_info_set_name (info, "/");
@@ -1180,14 +1189,14 @@ do_query_info (GVfsBackend *backend,
       g_file_info_set_display_name (info, _("Trash"));
       g_file_info_set_content_type (info, "inode/directory");
 
-      if (G_VFS_BACKEND_TRASH (backend)->top_files != NULL)
+      if (trash_backend->top_files != NULL)
         icon = g_themed_icon_new ("user-trash-full");
       else
         icon = g_themed_icon_new ("user-trash");
         
       g_file_info_set_icon (info, icon);
       g_object_unref (icon);
-      
+     
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_STANDARD_IS_VIRTUAL,
                                          TRUE);
@@ -1209,6 +1218,9 @@ do_query_info (GVfsBackend *backend,
       g_file_info_set_attribute_boolean (info,
                                          G_FILE_ATTRIBUTE_ACCESS_CAN_RENAME,
                                          FALSE);
+      g_file_info_set_attribute_uint32 (info,
+                                        G_FILE_ATTRIBUTE_TRASH_ITEM_COUNT,
+                                        trash_backend->num_top_files);
       
       g_vfs_job_succeeded (G_VFS_JOB (job));
     }
