@@ -34,7 +34,7 @@
 #include "gvfsjob.h"
 #include "gvfsjobsource.h"
 
-G_DEFINE_TYPE (GVfsJob, g_vfs_job, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GVfsJob, g_vfs_job, G_TYPE_OBJECT)
 
 /* TODO: Real P_() */
 #define P_(_x) (_x)
@@ -188,16 +188,35 @@ g_vfs_job_run (GVfsJob *job)
   GVfsJobClass *class;
 
   class = G_VFS_JOB_GET_CLASS (job);
+
+  /* Ensure that the job lives durint the whole
+   * lifetime of the call, as it may disappear when
+   * we call g_vfs_job_succeed/fail()
+   */
+  g_object_ref (job);
+  
   class->run (job);
+  
+  g_object_unref (job);
 }
 
 gboolean
 g_vfs_job_try (GVfsJob *job)
 {
   GVfsJobClass *class;
+  gboolean res;
 
   class = G_VFS_JOB_GET_CLASS (job);
-  return class->try (job);
+  
+  /* Ensure that the job lives during the whole
+   * lifetime of the call, as it may disappear when
+   * we call g_vfs_job_succeed/fail()
+   */
+  g_object_ref (job);
+  res = class->try (job);
+  g_object_unref (job);
+
+  return res;
 }
 
 void
@@ -244,8 +263,8 @@ g_vfs_job_failed (GVfsJob *job,
 }
 
 void
-g_vfs_job_failed_from_error (GVfsJob *job,
-			     GError *error)
+g_vfs_job_failed_from_error (GVfsJob *     job,
+			     const GError *error)
 {
   if (job->failed)
     return;
