@@ -49,6 +49,7 @@ g_vfs_job_mount_finalize (GObject *object)
 
   g_mount_spec_unref (job->mount_spec);
   g_object_unref (job->mount_source);
+  g_object_unref (job->backend);
   
   if (G_OBJECT_CLASS (g_vfs_job_mount_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_vfs_job_mount_parent_class)->finalize) (object);
@@ -86,7 +87,9 @@ g_vfs_job_mount_new (GMountSpec *spec,
   job->mount_spec = g_mount_spec_ref (spec);
   job->mount_source = g_object_ref (source);
   job->is_automount = is_automount;
-  job->backend = backend;
+  /* Ref the backend so we're sure its alive
+     during the whole job request. */
+  job->backend = g_object_ref (backend);
   if (request)
     job->request = dbus_message_ref (request);
   
@@ -97,7 +100,8 @@ static void
 run (GVfsJob *job)
 {
   GVfsJobMount *op_job = G_VFS_JOB_MOUNT (job);
-  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (op_job->backend);
+  GVfsBackend *backend = op_job->backend;
+  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (backend);
 
   if (class->mount == NULL)
     {
@@ -106,7 +110,7 @@ run (GVfsJob *job)
       return;
     }
   
-  class->mount (op_job->backend,
+  class->mount (backend,
 		op_job,
 		op_job->mount_spec,
 		op_job->mount_source,
@@ -117,16 +121,19 @@ static gboolean
 try (GVfsJob *job)
 {
   GVfsJobMount *op_job = G_VFS_JOB_MOUNT (job);
-  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (op_job->backend);
+  GVfsBackend *backend = op_job->backend;
+  GVfsBackendClass *class = G_VFS_BACKEND_GET_CLASS (backend);
+  gboolean result;
 
   if (class->try_mount == NULL)
     return FALSE;
 
-  return class->try_mount (op_job->backend,
-			   op_job,
-			   op_job->mount_spec,
-			   op_job->mount_source,
-			   op_job->is_automount);
+  result = class->try_mount (backend,
+			     op_job,
+			     op_job->mount_spec,
+			     op_job->mount_source,
+			     op_job->is_automount);
+  return result;
 }
 
 static void
@@ -150,7 +157,7 @@ mount_failed (GVfsJobMount *op_job, GError *error)
 	}
     }
   else
-    g_warning ("Mount failed: %s\n", error->message);
+    g_print ("Mount failed: %s\n", error->message);
 
   backend = g_object_ref (op_job->backend);
   g_vfs_job_emit_finished (G_VFS_JOB (op_job));
